require 'rails_helper'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods
  
  # Authentiicated controller
  describe Api::Dmo::HousingRequestSharesController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:admin) { create(:user, roles: ['Administrator']) }
    let!(:housing_request) { create(:active_request, documents_required: 10) }
    let!(:applicant) { housing_request.main_applicant }
    let!(:lessor) { create(:lessor) }


    describe 'POST /hermes/api/dmo/housing_request_shares' do
      context 'authenticated user' do

        before do
          authenticated_header(request, admin)
          post :create, data: { attributes: { housing_request_id: housing_request.id, lessor_id: lessor.id } }
        end

        it 'returns status code 201' do
          expect(response.status).to eq(201)
        end

        it 'returns housing_request_share values' do
          expect(json['data']['id']).not_to be_nil
          expect(json['data']['attributes']['code']).not_to be_nil
          expect(json['data']['attributes']['expire_at']).not_to be_nil
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
          expect(json['data']['attributes']['housing_request_id']).to eq(housing_request.id.to_s)
          expect(json['data']['attributes']['public_url']).not_to be_nil
        end

      end
    end
  end

  # Public controller
  describe Api::HousingRequestSharesController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:housing_request_share) { create(:housing_request_share, housing_request: FactoryBot.create(:active_request, documents_required: 10)) }
    let!(:housing_request) { housing_request_share.housing_request }
    let!(:applicant) { housing_request.main_applicant }
    let!(:lessor) { housing_request_share.lessor }

    describe 'GET /api/dmo/housing_request_shares/:id' do
      context 'with valid context' do
        it 'returns status code 200' do
          get :show, id: housing_request_share.id, code: housing_request_share.code
          expect(response.status).to eq(200)
          expect(json['included'].count).to be >= 1
          expect(json['included'].first['attributes'].keys).to include('required_documents')
          expect(json['included'].first['attributes']['required_documents']).to be_an_instance_of(Array)
        end
      end
      context 'with invalid context' do
        it 'should return 404 error whith invalid id' do
          get :show, id: 123, code: housing_request_share.code
          expect(response.status).to eq(404)
        end

        it 'should return 404 error whith invalid code' do
          get :show, id: housing_request_share.id, code: 'abcd'
          expect(response.status).to eq(404)
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  # Authenticated controller
  describe Api::Dmo::HousingRequestSharesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_REQUEST_SHARES = 'API_DMO_HOUSING_REQUEST_SHARES'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'has a admin role' do
      path '/api/dmo/housing_request_shares' do
        post 'create HousingRequestShare' do
          tags TAGS_API_DMO_HOUSING_REQUEST_SHARES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:housing_request_id] = { type: :string }
          json_params[:lessor_id] = { type: :string }

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing_request_share created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:housing_request) { create(:active_request, documents_required: 10) }
            let!(:applicant) { housing_request.main_applicant }
            let!(:lessor) { create(:lessor) }
            let(:params) do
              jsonapi_attributes({
                housing_request_id: housing_request.id,
                lessor_id: lessor.id
              })
            end

            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_request_shares'
            end

          end

          response '422', "missing or invalid required fields [:housing_request_id, :lessor_id]" do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:housing_request) { create(:active_request, documents_required: 10) }
            let!(:applicant) { housing_request.main_applicant }
            let!(:lessor) { create(:lessor) }
            let(:params) do
              jsonapi_attributes({
                housing_request_id: nil,
                lessor_id: nil
              })
            end
            run_test!
          end
        end
      end
    end
  end

  # Public controller
  describe Api::HousingRequestSharesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes

    context 'has a admin role' do
      let!(:housing_request_share) { create(:housing_request_share, housing_request: FactoryBot.create(:active_request, documents_required: 10)) }
      path '/api/dmo/housing_request_shares/{id}' do
        get 'get HousingRequestShare' do
          tags TAGS_API_DMO_HOUSING_REQUEST_SHARES
          produces 'application/json'
          consumes 'application/json'

          parameter name: :id, in: :path, type: :string,  required: true
          parameter name: :code, in: :query, type: :string, required: true

          response '200', 'housing_request_share found' do
            let!(:id) { housing_request_share.id }
            let!(:code) { housing_request_share.code }

            include_context 'with integration test'

            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data).to be_a Hash
              expect(json_data['id']).to eq(housing_request_share.housing_request.id.to_s)
            end
          end

          response '404', 'housing_request_share not found or invalid code' do
            let!(:id) { 123 }
            let!(:code) { 'abcde' }

            run_test!
          end

          response '498', 'housing_request_share expired' do
            before do
              housing_request_share.update(expire_at: DateTime.current - 1.day)
            end

            let!(:id) { housing_request_share.id }
            let!(:code) { housing_request_share.code }

            run_test!
          end
        end
      end
    end
  end
end
